library(matlab)  # calling for "meshgrid"
##############################################################
## File: DAR.R
##
## This program is for a double AR (DAR) model:
## Y_{t} = phi*Y_{t-1} +
##         varepsilon_{t}[ omega + alpha*Y^{2}_{t-1} ]^{1/2}
##
## Coded by: Dong Li 
##
## Reference:
## Li, D. and Qiu, J. (2020).
##   The marginal density of a TMA(1) process. 
##   Journal of Time Series Analysis, 41, 476--484.
##############################################################
##########  The function K(x, y)  ############################
##############################################################
K.fun = function(z,th){
f     = pnorm((z$x-th[1]*z$y)/sqrt(th[2]+th[3]*z$y^2))
return(f)
}
##############################################################
##############################################################
g.den = function(x,XX,G,th){
dif   = c(G,1)-c(0,G)
fden  = sum((dnorm((x-th[1]*XX)/sqrt(th[2]+th[3]*XX^2))/sqrt(th[2]+th[3]*XX^2))*dif)
return(fden)
}
##############################################################
th = c(1, 0.1, 0.1)  # parameters
x  = seq(-6,6,by=0.01)
n  = length(x)
y  = (c(min(x)-1,x)+c(x, max(x)+1))/2
z  = meshgrid(x,y)

A = t(K.fun(z,th))
K = A[,1:n]-A[,2:(n+1)]
a = A[,n+1]
DAR.G = solve(eye(n)-K)%*%a
DAR.g.den = sapply(x,g.den,XX=c(x,max(x)+1),G=DAR.G,th)

############# Plot the density function ##########################
par(mfrow=c(1,2))
plot(x,DAR.g.den,type="l",col="blue",las=1,lwd=3,cex.lab=1.3,cex.axis=1.3,
  xlab="x",ylab="",main=expression(paste("(a)  ", alpha==0.1),adj=0),
  cex.main=2,ylim=c(0,0.5))
abline(h=c(0,1),lty=2)

##############################################################
##############################################################
##############################################################

##############################################################
th = c(1,0.1,1)
x  = seq(-6,6,by=0.01)
n  = length(x)
y  = (c(min(x)-1,x)+c(x, max(x)+1))/2
z  = meshgrid(x,y)

A = t(K.fun(z,th))
K = A[,1:n]-A[,2:(n+1)]
a = A[,n+1]
DAR.G = solve(eye(n)-K)%*%a
DAR.g.den = sapply(x,g.den,XX=c(x,max(x)+1),G=DAR.G,th)

############# Plot the density function ##########################
#par(mfrow=c(1,2))
plot(x,DAR.g.den,type="l",col="red",las=1,lwd=3,cex.lab=1.3,cex.axis=1.3,
  xlab="x",ylab="",main=expression(paste("(b)  ", alpha==1),adj=0),
  cex.main=2,ylim=c(0,0.5))
abline(h=c(0,1),lty=2)


###################################################################